#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class PoliciesController < ApplicationController
  self.view_paths = 'lib/modules/ips/app/views'
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_action :set_sensor
  before_action :check_sensor_privileges, only: [:new, :destroy, :clone,
                                                 :global_update]
  before_action :check_license, only: [:new]

  before_action :set_policy, only: [:edit, :update, :destroy, :update_rule_db_versions, :clone]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def index
    @policies = if @sensor
                  @sensor.policies.includes(:rule_db_versions, :owner)
                  .where(is_deleted: false).order('LOWER(name) ASC')
                else
                  Policy.includes(:rule_db_versions, :owner)
                  .where(is_deleted: false).order('LOWER(name) ASC')
                end
  end

  def new
    @sensor = Sensor.find(params[:sensor_id])
  
    @rule_sources = get_rule_sources
  
    @policy = @sensor.owned_policies.build(
      default_enabled_rules: true,
      auto_resolve_flowbits: true,
      auto_add_new_rules: false
    )
  
    @owners = @policy.owner
      .self_and_ancestors_with_root
      .select { |o| current_user.can?(:manage, o) }
      .map { |o| [o.name, o.id] }
  
    render layout: false
  end
   

  def create
    @sensor = Sensor.find(params[:policy][:sensor_id])
  
    if current_user.cannot?(:manage, @sensor)
      redirect_to sensors_path, alert: I18n.t('controllers.no_privileges') and return
    end
  
    return error_404(I18n.t('controllers.sensor_no_license')) unless @sensor.can_assign_policy?
  
    @rule_sources = get_rule_sources
  
    @policy = Policy.new(policy_params)
  
    if @policy.save
      flash[:notice] = "#{I18n.t('controllers.signature_created')}"
    else
      @owners = @policy.owner
        .self_and_ancestors_with_root
        .select { |s| current_user.can?(:manage, s) }
        .map { |s| [s.name, s.id] }  
    end
  end
  

  def edit
    @owners = @policy.owner.self_and_ancestors_with_root
              .select { |s| current_user.can?(:manage, s) }.map { |s| [s.name, s.id] }

    if current_user.cannot? :manage, @policy.owner
      redirect_to sensors_path,
                  alert: "#{I18n.t('controllers.no_privileges')}"
    end

    render layout: false
  end

  def update
    @sensor = @policy.owner

    if current_user.cannot? :manage, @sensor
      redirect_to sensors_path,
                  alert: "#{I18n.t('controllers.no_privileges')}"
    end

    return unless @policy.update(policy_params)
    flash[:notice] = "#{I18n.t('controllers.signature_updated')}"

  end

  def search
    @search = params[:search]
    @policies = if @sensor
                  @sensor.policies.includes(:rule_db_versions, :owner)
                  .where(is_deleted: false)
                  .where('LOWER(name) LIKE ?', "%#{@search.downcase.split.join('%')}%")
                  .order('LOWER(name) ASC')
                else
                  Policy.includes(:rule_db_versions, :owner)
                  .where(is_deleted: false)
                  .where('LOWER(name) LIKE ?', "%#{@search.downcase.split.join('%')}%")
                  .order('LOWER(name) ASC')
                end
  end

  def destroy
    @policy.update(is_deleted: true)

    Delayed::Job.enqueue(Redborder::Jobs::DeletePolicyJob.new(@policy.id,
                                                              current_user.id,
                                                              true,
                                                              I18n.locale),
                         priority: 10, run_at: Time.now)

    redirect_to sensor_policies_path(@sensor),
                notice: "#{I18n.t('controllers.signature_deleted')}"
  end

  def update_rule_db_versions
    if current_user.cannot? :manage, @policy.owner
      redirect_to sensors_path,
                  alert: "#{I18n.t('controllers.no_privileges')}" and return
    end

    if @policy.updating
      redirect_to sensors_path,
                  alert: "#{I18n.t('controllers.the_signature')} #{@policy.name} #{I18n.t('controllers.is_updating')}"
    elsif @policy.block_updates
      redirect_to sensors_path,
                  alert: "#{I18n.t('controllers.the_signature')} #{@policy.name} #{I18n.t('controllers.is_blocked')}"
    else
      @policy.update(updating: true)
      Delayed::Job.enqueue(
        Redborder::Jobs::PolicyUpdateRuleDbVersionsJob.new(@policy.id,
                                                           current_user.id,
                                                           true,
                                                           I18n.locale),
        priority: 1,
        run_at: Time.now + 10.seconds
      )

      redirect_to sensors_path
    end
  end

  def clone
    new_policy = @policy.clone @sensor.id

    Delayed::Job.enqueue(
      Redborder::Jobs::ClonePolicyJob.new(true,
                                          @policy.id,
                                          new_policy.id,
                                          current_user.id,
                                          I18n.locale),
      priority: 5,
      run_at: Time.now + 5.seconds
    )

    redirect_to sensor_policies_path(@sensor)
  end

  #
  # Apply an update to selected policies.
  #
  def global_update
    @sensor = Sensor.find(params[:sensor_id])
    available_ids = @sensor.self_and_descendants.map(&:id)

    if request.get?
      @policies = Policy
                  .where(updating: false, sensor_id: available_ids)
                  .select(&:update_available?)
    elsif request.patch?
      policies = Policy
                 .where(id: params[:policy_ids], updating: false)
                 .select { |s| available_ids.include?(s.id) }
                 .each { |p| p.update(updating: true) }

      policies.each_with_index do |policy, i|
        Delayed::Job.enqueue(
          Redborder::Jobs::PolicyUpdateRuleDbVersionsJob.new(policy.id,
                                                             current_user.id,
                                                             true,
                                                             I18n.locale),
          priority: 1,
          run_at: Time.now + (i * 5).seconds
        )
      end
      redirect_to sensors_path
    end
  end

  private

  # Use callbacks to share common setup or constraints between actions.
  def set_sensor
    @sensor = Sensor.find(params[:sensor_id]) if params[:sensor_id]
  end

  def set_policy
    @policy = Policy.find(params[:id])
    return error_403("#{I18n.t('controllers.policy_updating')}") if @policy.updating
  end

  def policy_params
    params.require(:policy)
      .permit(:sensor_id, :name, :description,
              :default_enabled_rules, :auto_resolve_flowbits,
              :auto_add_new_rules, :block_updates, :default_action,
              rule_db_version_ids: [])
  end

  #
  # Check sensor permissions or redirect with an error.
  #
  def check_sensor_privileges
    return if current_user.admin? && current_user.can?(:manage, @sensor)
    redirect_to sensors_path, alert: "#{I18n.t('controllers.no_privileges')}"
  end

  def check_license
    return error_404("#{I18n.t('controllers.sensor_no_license')}") unless @sensor.can_assign_policy?
  end

  def get_rule_sources
    RuleSource
    .joins(:rule_db_versions)
    .where(sensor_id: current_user.sensor.tree(true))
    .where(rule_db_versions: { completed: true, checked: true })
    .then { |scope|
      if @sensor.snort3_sensor?
        scope.where("snort_version LIKE ?", 'snort3%')
      elsif @sensor.snort2_sensor?
        scope.where("snort_version NOT LIKE ?", 'snort3%')
      else
        scope
      end
    }.distinct
  end
end
